@@ -35,7 +35,6 @@
 #include "connection.h"
 #include "debug.h"
 #include "disco.h"
-#include "gabble-signals-marshal.h"
 #include "namespaces.h"
 #include "util.h"
 
@@ -47,16 +46,6 @@ G_DEFINE_TYPE_WITH_CODE (GabbleBytestreamIBB, gabble_bytestream_ibb,
     G_IMPLEMENT_INTERFACE (GABBLE_TYPE_BYTESTREAM_IFACE,
       bytestream_iface_init));
 
-/* signals */
-enum
-{
-  DATA_RECEIVED,
-  STATE_CHANGED,
-  LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = {0};
-
 /* properties */
 enum
 {
@@ -217,7 +206,7 @@ gabble_bytestream_ibb_set_property (GObject *object,
         if (priv->state != g_value_get_uint (value))
             {
               priv->state = g_value_get_uint (value);
-              g_signal_emit (object, signals[STATE_CHANGED], 0, priv->state);
+              g_signal_emit_by_name (object, "state-changed", priv->state);
             }
         break;
       case PROP_BLOCK_SIZE:
@@ -321,24 +310,6 @@ gabble_bytestream_ibb_class_init (
       G_PARAM_CONSTRUCT | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_BLOCK_SIZE,
       param_spec);
-
-  signals[DATA_RECEIVED] =
-    g_signal_new ("data-received",
-                  G_OBJECT_CLASS_TYPE (gabble_bytestream_ibb_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__UINT_POINTER,
-                  G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_POINTER);
-
-  signals[STATE_CHANGED] =
-    g_signal_new ("state-changed",
-                  G_OBJECT_CLASS_TYPE (gabble_bytestream_ibb_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  gabble_marshal_VOID__UINT,
-                  G_TYPE_NONE, 1, G_TYPE_UINT);
 }
 
 /*
@@ -481,7 +452,7 @@ gabble_bytestream_ibb_receive (GabbleBytestreamIBB *self,
       return;
     }
 
-  g_signal_emit (G_OBJECT (self), signals[DATA_RECEIVED], 0, sender, str);
+  g_signal_emit_by_name (G_OBJECT (self), "data-received", sender, str);
   g_string_free (str, TRUE);
 
   if (is_iq)
