@@ -152,12 +152,26 @@ struct _GabbleMediaChannelPrivate
   TpLocalHoldState hold_state;
   TpLocalHoldStateReason hold_state_reason;
 
+  GPtrArray *delayed_request_streams;
+
   /* These are really booleans, but gboolean is signed. Thanks, GLib */
   unsigned ready:1;
   unsigned closed:1;
   unsigned dispose_has_run:1;
 };
 
+struct _delayed_request_streams_ctx {
+  GabbleMediaChannel *chan;
+  gulong caps_disco_id;
+  guint timeout_id;
+  guint contact_handle;
+  GArray *types;
+  DBusGMethodInvocation *context;
+};
+
+static void destroy_request (struct _delayed_request_streams_ctx *ctx,
+    gpointer user_data);
+
 #define GABBLE_MEDIA_CHANNEL_GET_PRIVATE(obj) (obj->priv)
 
 static void
@@ -183,7 +197,8 @@ static void session_new_content_cb (GabbleJingleSession *session,
     GabbleJingleContent *c, gpointer user_data);
 static GabbleMediaStream *
 create_stream_from_content (GabbleMediaChannel *chan, GabbleJingleContent *c);
-static gboolean contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer);
+static gboolean contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer,
+    gboolean *wait);
 
 static gboolean
 _create_streams (GabbleMediaChannel *chan)
@@ -696,6 +711,13 @@ gabble_media_channel_dispose (GObject *object)
 
   priv->dispose_has_run = TRUE;
 
+  if (priv->delayed_request_streams != NULL)
+    {
+      g_ptr_array_foreach (priv->delayed_request_streams,
+          (GFunc) destroy_request, NULL);
+      g_assert (priv->delayed_request_streams == NULL);
+    }
+
   tp_handle_unref (contact_handles, priv->creator);
   priv->creator = 0;
 
@@ -1300,7 +1322,7 @@ _gabble_media_channel_request_streams (GabbleMediaChannel *chan,
   g_object_get (priv->session, "peer", &peer,
       "peer-resource", &peer_resource, NULL);
 
-  if (!contact_is_media_capable (chan, peer))
+  if (!contact_is_media_capable (chan, peer, NULL))
     {
       DEBUG ("peer has no a/v capabilities");
       g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
@@ -1443,6 +1465,105 @@ _gabble_media_channel_request_streams (GabbleMediaChannel *chan,
   return TRUE;
 }
 
+/* user_data param is here so we match the GFunc prototype */
+static void
+destroy_request (struct _delayed_request_streams_ctx *ctx,
+    gpointer user_data G_GNUC_UNUSED)
+{
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (ctx->chan);
+
+  if (ctx->timeout_id)
+    g_source_remove (ctx->timeout_id);
+
+  if (ctx->caps_disco_id)
+    g_signal_handler_disconnect (priv->conn->presence_cache,
+        ctx->caps_disco_id);
+
+  if (ctx->context != NULL)
+    {
+      GError *error = NULL;
+      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "cannot add streams: peer has insufficient caps");
+      dbus_g_method_return_error (ctx->context, error);
+      g_error_free (error);
+    }
+
+  g_array_free (ctx->types, TRUE);
+  g_slice_free (struct _delayed_request_streams_ctx, ctx);
+  g_ptr_array_remove_fast (priv->delayed_request_streams, ctx);
+
+  if (priv->delayed_request_streams->len == 0)
+    {
+      g_ptr_array_free (priv->delayed_request_streams, TRUE);
+      priv->delayed_request_streams = NULL;
+    }
+}
+
+static void gabble_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
+    guint contact_handle, const GArray *types, DBusGMethodInvocation *context);
+
+static gboolean
+repeat_request (struct _delayed_request_streams_ctx *ctx)
+{
+  gabble_media_channel_request_streams (
+      TP_SVC_CHANNEL_TYPE_STREAMED_MEDIA (ctx->chan),
+      ctx->contact_handle, ctx->types, ctx->context);
+
+  ctx->timeout_id = 0;
+  ctx->context = NULL;
+  destroy_request (ctx, NULL);
+  return FALSE;
+}
+
+static void
+capabilities_discovered_cb (GabblePresenceCache *cache,
+                            TpHandle handle,
+                            struct _delayed_request_streams_ctx *ctx)
+{
+  /* If there are more cache caps pending, wait for them. */
+  if (gabble_presence_cache_caps_pending (cache, handle))
+    return;
+
+  repeat_request (ctx);
+}
+
+static void
+delay_stream_request (GabbleMediaChannel *chan,
+                      TpSvcChannelTypeStreamedMedia *iface,
+                      guint contact_handle,
+                      const GArray *types,
+                      DBusGMethodInvocation *context,
+                      gboolean disco_in_progress)
+{
+  GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
+  struct _delayed_request_streams_ctx *ctx =
+    g_slice_new0 (struct _delayed_request_streams_ctx);
+
+  ctx->chan = chan;
+  ctx->contact_handle = contact_handle;
+  ctx->context = context;
+  ctx->types = g_array_sized_new (FALSE, FALSE, sizeof(guint), types->len);
+  g_array_append_vals (ctx->types, types->data, types->len);
+
+  if (disco_in_progress)
+    {
+      ctx->caps_disco_id = g_signal_connect (priv->conn->presence_cache,
+          "capabilities-discovered", G_CALLBACK (capabilities_discovered_cb),
+          ctx);
+      ctx->timeout_id = 0;
+    }
+  else
+    {
+      ctx->caps_disco_id = 0;
+      ctx->timeout_id = g_timeout_add_seconds (5,
+          (GSourceFunc) repeat_request, ctx);
+    }
+
+  if (priv->delayed_request_streams == NULL)
+      priv->delayed_request_streams = g_ptr_array_sized_new (1);
+
+  g_ptr_array_add (priv->delayed_request_streams, ctx);
+}
 
 /**
  * gabble_media_channel_request_streams
@@ -1463,6 +1584,7 @@ gabble_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
   GError *error = NULL;
   GPtrArray *ret;
   TpHandleRepoIface *contact_handles;
+  gboolean wait;
 
   g_assert (GABBLE_IS_MEDIA_CHANNEL (self));
 
@@ -1476,6 +1598,29 @@ gabble_media_channel_request_streams (TpSvcChannelTypeStreamedMedia *iface,
   if (!tp_handle_is_valid (contact_handles, contact_handle, &error))
     goto error;
 
+  /* If we know the caps haven't arrived yet, delay stream creation
+   * and check again later */
+  if (!contact_is_media_capable (self, contact_handle, &wait))
+    {
+      if (wait)
+        {
+          DEBUG ("Delaying RequestStreams until we get all caps from contact");
+          delay_stream_request (self, iface, contact_handle, types, context,
+              TRUE);
+          return;
+        }
+
+      /* if we're unsure about the offlineness of the contact, wait a bit */
+      if (gabble_presence_cache_is_unsure (priv->conn->presence_cache))
+        {
+          DEBUG ("Delaying RequestStreams because we're unsure about them");
+          delay_stream_request (self, iface, contact_handle, types, context,
+              FALSE);
+          return;
+        }
+    }
+
+
   if (priv->session == NULL)
     {
       create_session (self, contact_handle);
@@ -1516,7 +1661,7 @@ error:
 
 
 static gboolean
-contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer)
+contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer, gboolean *wait)
 {
   GabbleMediaChannelPrivate *priv = GABBLE_MEDIA_CHANNEL_GET_PRIVATE (chan);
   GabblePresence *presence;
@@ -1527,6 +1672,16 @@ contact_is_media_capable (GabbleMediaChannel *chan, TpHandle peer)
       conn, TP_HANDLE_TYPE_CONTACT);
 #endif
 
+  if (wait != NULL)
+    *wait = FALSE;
+
+  if (gabble_presence_cache_caps_pending (priv->conn->presence_cache, peer))
+    {
+      DEBUG ("caps are pending for peer %u", peer);
+      if (wait != NULL)
+        *wait = TRUE;
+    }
+
   caps = PRESENCE_CAP_GOOGLE_VOICE | PRESENCE_CAP_JINGLE_RTP |
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO | PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO;
 
@@ -1563,6 +1718,7 @@ _gabble_media_channel_add_member (GObject *obj,
   if (priv->creator == mixin->self_handle)
     {
       TpIntSet *set;
+      gboolean wait;
 
       /* yes: check we don't have a peer already, invite this onis one */
 
@@ -1581,12 +1737,23 @@ _gabble_media_channel_add_member (GObject *obj,
             }
         }
 
-      if (!contact_is_media_capable (chan, handle))
+      /* We can't delay the request at this time, but if there's a chance
+       * the caps might be available later, we'll add the contact and
+       * hope for the best. */
+      if (!contact_is_media_capable (chan, handle, &wait))
         {
-          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-              "handle %u cannot be added: has no media capabilities",
-              handle);
-          return FALSE;
+          if (wait ||
+              gabble_presence_cache_is_unsure (priv->conn->presence_cache))
+            {
+              DEBUG ("contact %u caps still pending, adding anyways", handle);
+            }
+          else
+            {
+              g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+                  "handle %u cannot be added: has no media capabilities",
+                  handle);
+              return FALSE;
+            }
         }
 
       /* make the peer remote pending */
