@@ -358,12 +358,15 @@ d_bus_names_changed_removed (GabbleTubesChannel *self,
                              TpHandle contact)
 {
   GabbleTubesChannelPrivate *priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
-  GPtrArray *added = g_ptr_array_new ();
-  GArray *removed = g_array_new (FALSE, FALSE, sizeof (guint));
+  GPtrArray *added;
+  GArray *removed;
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     return;
 
+  added = g_ptr_array_new ();
+  removed = g_array_new (FALSE, FALSE, sizeof (guint));
+
   g_array_append_val (removed, contact);
 
   tp_svc_channel_type_tubes_emit_d_bus_names_changed (self,
@@ -831,6 +834,7 @@ gabble_tubes_channel_presence_updated (GabbleTubesChannel *self,
 
               /* the tube has reffed its initiator, no need to keep a ref */
               tp_handle_unref (contact_repo, initiator_handle);
+              g_hash_table_destroy (parameters);
             }
         }
       else
@@ -989,23 +993,6 @@ gabble_tubes_channel_list_tubes (TpSvcChannelTypeTubes *iface,
   g_ptr_array_free (ret, TRUE);
 }
 
-static void
-copy_parameter (gpointer key,
-                gpointer value,
-                gpointer user_data)
-{
-  const gchar *prop = key;
-  GValue *gvalue = value;
-  GHashTable *parameters = user_data;
-  GValue *gvalue_copied;
-
-  gvalue_copied = g_slice_new0 (GValue);
-  g_value_init (gvalue_copied, G_VALUE_TYPE (gvalue));
-  g_value_copy (gvalue, gvalue_copied);
-
-  g_hash_table_insert (parameters, g_strdup (prop), gvalue_copied);
-}
-
 struct _i_hate_g_hash_table_foreach
 {
   GabbleTubesChannel *self;
@@ -1172,6 +1159,7 @@ gabble_tubes_channel_tube_si_offered (GabbleTubesChannel *self,
 
       NODE_DEBUG (tube_node, e.message);
       gabble_bytestream_iface_close (bytestream, &e);
+      g_hash_table_destroy (parameters);
       return;
     }
 
@@ -1187,6 +1175,8 @@ gabble_tubes_channel_tube_si_offered (GabbleTubesChannel *self,
 
   tube = create_new_tube (self, type, priv->handle, service,
       parameters, stream_id, tube_id, (GabbleBytestreamIface *) bytestream);
+
+  g_hash_table_destroy (parameters);
 }
 
 /* Called when we receive a SI request,
@@ -1448,6 +1438,8 @@ tube_msg_offered (GabbleTubesChannel *self,
 
   tp_channel_manager_emit_new_channel (priv->conn->private_tubes_factory,
       TP_EXPORTABLE_CHANNEL (tube), NULL);
+
+  g_hash_table_destroy (parameters);
 }
 
 static void
@@ -1603,7 +1595,6 @@ gabble_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
   TpBaseConnection *base;
   guint tube_id;
   GabbleTubeIface *tube;
-  GHashTable *parameters_copied;
   gchar *stream_id;
 
   g_assert (GABBLE_IS_TUBES_CHANNEL (self));
@@ -1611,15 +1602,11 @@ gabble_tubes_channel_offer_d_bus_tube (TpSvcChannelTypeTubes *iface,
   priv = GABBLE_TUBES_CHANNEL_GET_PRIVATE (self);
   base = (TpBaseConnection *) priv->conn;
 
-  parameters_copied = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
-      (GDestroyNotify) tp_g_value_slice_free);
-  g_hash_table_foreach (parameters, copy_parameter, parameters_copied);
-
   stream_id = gabble_bytestream_factory_generate_stream_id ();
   tube_id = generate_tube_id ();
 
   tube = create_new_tube (self, TP_TUBE_TYPE_DBUS, priv->self_handle,
-      service, parameters_copied, (const gchar *) stream_id, tube_id, NULL);
+      service, parameters, (const gchar *) stream_id, tube_id, NULL);
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
@@ -1684,7 +1671,6 @@ gabble_tubes_channel_offer_stream_tube (TpSvcChannelTypeTubes *iface,
   TpBaseConnection *base;
   guint tube_id;
   GabbleTubeIface *tube;
-  GHashTable *parameters_copied;
   gchar *stream_id;
   GError *error = NULL;
 
@@ -1701,15 +1687,11 @@ gabble_tubes_channel_offer_stream_tube (TpSvcChannelTypeTubes *iface,
       return;
     }
 
-  parameters_copied = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
-      (GDestroyNotify) tp_g_value_slice_free);
-  g_hash_table_foreach (parameters, copy_parameter, parameters_copied);
-
   stream_id = gabble_bytestream_factory_generate_stream_id ();
   tube_id = generate_tube_id ();
 
   tube = create_new_tube (self, TP_TUBE_TYPE_STREAM, priv->self_handle,
-      service, parameters_copied, (const gchar *) stream_id, tube_id, NULL);
+      service, parameters, (const gchar *) stream_id, tube_id, NULL);
 
   g_object_set (tube,
       "address-type", address_type,
