@@ -43,6 +43,14 @@ typedef struct _GabbleOlpcView GabbleOlpcView;
 typedef struct _GabbleOlpcGadgetManager GabbleOlpcGadgetManager;
 
 
+typedef struct _GabbleJingleFactory GabbleJingleFactory;
+typedef struct _GabbleJingleSession GabbleJingleSession;
+typedef struct _GabbleJingleContent GabbleJingleContent;
+typedef struct _GabbleJingleTransportGoogle GabbleJingleTransportGoogle;
+typedef struct _GabbleJingleMediaRtp GabbleJingleMediaRtp;
+
+typedef struct _JingleCandidate JingleCandidate;
+
 typedef enum {
     INITIATOR_INVALID = -1,
     INITIATOR_LOCAL = 0,
@@ -53,7 +61,7 @@ typedef enum {
     PRESENCE_CAP_NONE = 0,
     PRESENCE_CAP_GOOGLE_TRANSPORT_P2P = 1 << 0,
     PRESENCE_CAP_GOOGLE_VOICE = 1 << 1,
-    PRESENCE_CAP_JINGLE = 1 << 2,
+    PRESENCE_CAP_JINGLE015 = 1 << 2,
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO = 1 << 3,
     PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO = 1 << 4,
     PRESENCE_CAP_CHAT_STATES = 1 << 5,
@@ -62,7 +70,11 @@ typedef enum {
     PRESENCE_CAP_IBB = 1 << 8,
     PRESENCE_CAP_SI_TUBES = 1 << 9,
     PRESENCE_CAP_FILE_TRANSFER = 1 << 10,
-    PRESENCE_CAP_OLPC_1 = 1 << 11
+    PRESENCE_CAP_OLPC_1 = 1 << 11,
+    PRESENCE_CAP_JINGLE_RTP = 1 << 12,
+    PRESENCE_CAP_JINGLE032 = 1 << 13,
+    PRESENCE_CAP_JINGLE_TRANSPORT_ICE = 1 << 14,
+    PRESENCE_CAP_JINGLE_TRANSPORT_RAWUDP = 1 << 15,
 } GabblePresenceCapabilities;
 
 G_END_DECLS
